@@ -28,51 +28,25 @@ def cli():
 @click.argument("datadir", type=click.Path(exists=True))
 @click.argument("region", type=str)
 @click.argument("outdir", type=click.Path())
-@click.option(
-    "-n",
-    "--nlo-method",
-    type=str,
-    default="DR",
-    help="tW simluation NLO method",
-    show_default=True,
-)
-@click.option(
-    "-x", "--override-selection", type=str, help="override selection with contents of file"
-)
+@click.option("-p", "--pre-exec", type=click.Path(resolve_path=True), help="Python code to pre-execute")
+@click.option("-n", "--nlo-method", type=str, default="DR", help="tW simluation NLO method", show_default=True)
+@click.option("-x", "--override-selection", type=str, help="override selection with contents of file")
 @click.option("-t", "--use-tptrw", is_flag=True, help="apply top pt reweighting")
 @click.option("-i", "--ignore-list", type=str, help="variable ignore list file")
-@click.option(
-    "-s",
-    "--test-size",
-    type=float,
-    default=0.5,
-    help="training test size",
-    show_default=True,
-)
-@click.option(
-    "-e",
-    "--early-stop",
-    type=int,
-    default=10,
-    help="number of early stopping rounds",
-    show_default=True,
-)
-@click.option("-u", "--use-dilep", is_flag=True, help="train with dilepton samples")
-@click.option(
-    "--learning-rate", type=float, required=True, help="learning_rate model parameter"
-)
-@click.option("--num-leaves", type=int, required=True, help="num_leaves model parameter")
-@click.option(
-    "--min-child-samples", type=int, required=True, help="min_child_samples model parameter"
-)
-@click.option("--max-depth", type=int, required=True, help="max_depth model parameter")
-@click.option(
-    "--n-estimators", type=int, required=True, help="n_estimators model parameter"
-)
+@click.option("-s", "--test-size", type=float, default=0.40, help="training test size", show_default=True)
+@click.option("-e", "--early-stop", type=int, default=10, help="number of early stopping rounds", show_default=True)
+@click.option("-d", "--use-dilep", is_flag=True, help="train with dilepton samples")
+@click.option("-k", "--use-sklearn", is_flag=True, help="use sklearn instead of lgbm")
+@click.option("--learning-rate", type=float, default=0.1, help="learning_rate model parameter", show_default=True)
+@click.option("--num-leaves", type=int, default=31, help="num_leaves model parameter", show_default=True)
+@click.option("--min-child-samples", type=int, default=50, help="min_child_samples model parameter", show_default=True)
+@click.option("--max-depth", type=int, default=5, help="max_depth model parameter", show_default=True)
+@click.option("--reg-lambda", type=float, default=0, help="lambda (L2) regularization", show_default=True)
 def single(
     datadir,
     region,
     outdir,
+    pre_exec,
     nlo_method,
     override_selection,
     use_tptrw,
@@ -80,14 +54,17 @@ def single(
     test_size,
     early_stop,
     use_dilep,
+    use_sklearn,
     learning_rate,
     num_leaves,
     min_child_samples,
     max_depth,
-    n_estimators,
+    reg_lambda,
 ):
     """Execute a single training round."""
-    # fmt: on
+    if pre_exec is not None:
+        exec(PosixPath(pre_exec).read_text())
+
     from tdub.ml_train import single_training, prepare_from_root
     from tdub.data import avoids_for, quick_files
     from tdub.frames import drop_cols
@@ -110,23 +87,24 @@ def single(
     if ignore_list:
         drops = PosixPath(ignore_list).read_text().strip().split()
         drop_cols(df, *drops)
-    params = dict(
+    train_axes = dict(
         learning_rate=learning_rate,
         num_leaves=num_leaves,
         min_child_samples=min_child_samples,
         max_depth=max_depth,
-        n_estimators=n_estimators,
+        reg_lambda=reg_lambda,
     )
     extra_sum = {"region": region, "nlo_method": nlo_method}
     single_training(
         df,
         y,
         w,
-        params,
+        train_axes,
         outdir,
         test_size=test_size,
         early_stopping_rounds=early_stop,
         extra_summary_entries=extra_sum,
+        use_sklearn=use_sklearn,
     )
     return 0
 
@@ -135,44 +113,27 @@ def single(
 @click.argument("datadir", type=click.Path(exists=True, resolve_path=True))
 @click.argument("region", type=str)
 @click.argument("workspace", type=click.Path(exists=False))
-@click.option(
-    "-n",
-    "--nlo-method",
-    type=str,
-    default="DR",
-    help="tW simluation NLO method",
-    show_default=True,
-)
-@click.option(
-    "-e",
-    "--early-stop",
-    type=int,
-    default=10,
-    help="number of early stopping rounds",
-    show_default=True,
-)
-@click.option(
-    "-x", "--override-selection", type=str, help="override selection with contents of file"
-)
+@click.option("-p", "--pre-exec", type=click.Path(resolve_path=True), help="Python code to pre-execute")
+@click.option("-n", "--nlo-method", type=str, default="DR", help="tW simluation NLO method", show_default=True)
+@click.option("-e", "--early-stop", type=int, default=10, help="number of early stopping rounds", show_default=True)
+@click.option("-x", "--override-selection", type=str, help="override selection with contents of file")
 @click.option("-t", "--use-tptrw", is_flag=True, help="apply top pt reweighting")
 @click.option("-i", "--ignore-list", type=str, help="variable ignore list file")
-@click.option("-u", "--use-dilep", is_flag=True, help="train with dilepton samples")
+@click.option("-s", "--test-size", type=float, default=0.40, help="training test size", show_default=True)
+@click.option("-d", "--use-dilep", is_flag=True, help="train with dilepton samples")
 @click.option("--overwrite", is_flag=True, help="overwrite existing workspace")
 @click.option("--and-submit", is_flag=True, help="submit the condor jobs")
-@click.option(
-    "--scan-params",
-    type=click.Path(exists=True),
-    help="override default scan parameters with json file",
-)
 def scan(
     datadir,
     region,
     workspace,
+    pre_exec,
     nlo_method,
     early_stop,
     override_selection,
     use_tptrw,
     ignore_list,
+    test_size,
     use_dilep,
     overwrite,
     and_submit,
@@ -187,23 +148,24 @@ def scan(
 
     """
 
-    from tdub.batch import (
-        create_condor_workspace,
-        condor_preamble,
-        add_condor_arguments,
-        condor_submit,
-    )
-    from tdub.config import DEFAULT_SCAN_PARAMETERS
+    if pre_exec is not None:
+        exec(PosixPath(pre_exec).read_text())
 
-    if scan_params is None:
-        pd = DEFAULT_SCAN_PARAMETERS
-    else:
-        pd = json.loads(PosixPath(scan_params).read_text())
+    # from tdub.batch import (
+    #     create_condor_workspace,
+    #     condor_preamble,
+    #     add_condor_arguments,
+    #     condor_submit,
+    # )
+
+    from tdub.batch import create_condor_workspace
+    import tdub.config
+    import itertools
 
     ws = create_condor_workspace(workspace, overwrite=overwrite)
     (ws / "res").mkdir()
 
-    log.info(f"Preparing a scan in {workspace}")
+    log.info(f"Preparing a scan; results saved to {workspace}")
     log.info(f"  - region: {region}")
     log.info(f"  - NLO method: {nlo_method}")
     log.info("  - Using {} samples".format("dilepton" if use_dilep else "inclusive"))
@@ -220,29 +182,32 @@ def scan(
         ignore_list = "_NONE"
     else:
         ignore_list = str(PosixPath(ignore_list).resolve())
+    if pre_exec is None:
+        pre_exec = "_NONE"
+    else:
+        pre_exec = str(PosixPath(pre_exec).resolve())
 
-    import itertools
-
+    pd = tdub.config.DEFAULT_SCAN_PARAMETERS
     itr = itertools.product(
         pd.get("max_depth"),
         pd.get("num_leaves"),
         pd.get("learning_rate"),
         pd.get("min_child_samples"),
-        pd.get("n_estimators"),
+        pd.get("reg_lambda"),
     )
 
-    for (max_depth, num_leaves, learning_rate, min_child_samples, n_estimators) in itr:
+    for (max_depth, num_leaves, learning_rate, min_child_samples, reg_lambda) in itr:
         suffix = "{}-{}-{}-{}-{}".format(
-            max_depth, num_leaves, learning_rate, min_child_samples, n_estimators,
+            max_depth, num_leaves, learning_rate, min_child_samples, reg_lambda,
         )
         outdir = ws / "res" / f"{i:04d}_{suffix}"
         arglist = (
-            "{} {} {} -n {} -x {} -i {} "
+            "{} {} {} -s {} -n {} -x {} -i {} -p {} "
             "--learning-rate {} "
             "--num-leaves {} "
             "--min-child-samples {} "
             "--max-depth {} "
-            "--n-estimators {} "
+            "--reg-lambda {} "
             "--early-stop {} "
             "{}"
             "{}"
@@ -250,20 +215,23 @@ def scan(
             datadir,
             region,
             outdir,
+            test_size,
             nlo_method,
             override_sel,
             ignore_list,
+            pre_exec,
             learning_rate,
             num_leaves,
             min_child_samples,
             max_depth,
-            n_estimators,
+            reg_lambda,
             early_stop,
             "-t " if use_tptrw else "",
-            "-u " if use_dilep else "",
+            "-d " if use_dilep else "",
         )
         arglist = arglist.replace("-x _NONE ", "")
         arglist = arglist.replace("-i _NONE ", "")
+        arglist = arglist.replace("-p _NONE ", "")
         runs.append(arglist)
         i += 1
 
@@ -326,17 +294,10 @@ def scan(
 @cli.command("train-check", context_settings=dict(max_content_width=92))
 @click.argument("workspace", type=click.Path(exists=True))
 @click.option("-p", "--print-top", is_flag=True, help="Print the top results")
-@click.option(
-    "-n",
-    "--n-res",
-    type=int,
-    default=10,
-    help="Number of top results to print",
-    show_default=True,
-)
+@click.option("-n", "--n-res", type=int, default=10, help="Number of top results to print", show_default=True)
 def check(workspace, print_top, n_res):
     """Check the results of a parameter scan WORKSPACE."""
-    from tdub.ml_train import SingleTrainingResult
+    from tdub.ml_train import SingleTrainingSummary
     import shutil
 
     results = []
@@ -352,7 +313,7 @@ def check(workspace, print_top, n_res):
             summary = json.load(f)
             if summary["bad_ks"]:
                 continue
-            res = SingleTrainingResult(**summary)
+            res = SingleTrainingSummary(**summary)
             res.workspace = resdir
             res.summary = summary
             results.append(res)
@@ -386,22 +347,8 @@ def check(workspace, print_top, n_res):
 @click.argument("scandir", type=click.Path(exists=True))
 @click.argument("datadir", type=click.Path(exists=True))
 @click.option("-t", "--use-tptrw", is_flag=True, help="use top pt reweighting")
-@click.option(
-    "-r",
-    "--random-seed",
-    type=int,
-    default=414,
-    help="random seed for folding",
-    show_default=True,
-)
-@click.option(
-    "-n",
-    "--n-splits",
-    type=int,
-    default=3,
-    help="number of splits for folding",
-    show_default=True,
-)
+@click.option("-r", "--random-seed", type=int, default=414, help="random seed for folding", show_default=True)
+@click.option("-n", "--n-splits", type=int, default=3, help="number of splits for folding", show_default=True)
 def fold(scandir, datadir, use_tptrw, random_seed, n_splits):
     """Perform a folded training based on a hyperparameter scan result."""
     from tdub.ml_train import folded_training, prepare_from_root
@@ -441,7 +388,7 @@ def fold(scandir, datadir, use_tptrw, random_seed, n_splits):
         {"verbose": 10},
         str(outdir),
         summary["region"],
-        kfold_kw={"n_splits": n_splits, "shuffle": True, "random_state": random_seed,},
+        kfold_kw={"n_splits": n_splits, "shuffle": True, "random_state": random_seed},
     )
     return 0
 
@@ -450,20 +397,8 @@ def fold(scandir, datadir, use_tptrw, random_seed, n_splits):
 @click.argument("infile", type=click.Path(exists=True))
 @click.argument("arrname", type=str)
 @click.argument("outdir", type=click.Path())
-@click.option(
-    "-f",
-    "--fold-results",
-    type=click.Path(exists=True),
-    multiple=True,
-    help="fold output directories",
-)
-@click.option(
-    "-s",
-    "--single-results",
-    type=click.Path(exists=True),
-    multiple=True,
-    help="single result dirs",
-)
+@click.option("-f", "--fold-results", type=click.Path(exists=True), multiple=True, help="fold output directories")
+@click.option("-s", "--single-results", type=click.Path(exists=True), multiple=True, help="single result dirs")
 def apply_single(infile, arrname, outdir, fold_results=None, single_results=None):
     """Generate BDT response array for INFILE and save to .npy file.
 
@@ -476,7 +411,7 @@ def apply_single(infile, arrname, outdir, fold_results=None, single_results=None
 
     from tdub.ml_apply import build_array, FoldedTrainSummary, SingleTrainSummary
     from tdub.data import SampleInfo
-    from tdub.branches import minimal_branches
+    from tdub.data import selection_branches
     from tdub.frames import raw_dataframe
     import numpy as np
 
@@ -494,7 +429,7 @@ def apply_single(infile, arrname, outdir, fold_results=None, single_results=None
     necessary_branches = ["OS", "elmu", "reg2j1b", "reg2j2b", "reg1j1b"]
     for res in trs:
         necessary_branches += res.features
-        necessary_branches += minimal_branches(res.selection_used)
+        necessary_branches += selection_branches(res.selection_used)
     necessary_branches = sorted(set(necessary_branches), key=str.lower)
 
     log.info("Loading necessary branches:")
@@ -516,24 +451,21 @@ def apply_single(infile, arrname, outdir, fold_results=None, single_results=None
 @click.argument("arrname", type=str)
 @click.argument("outdir", type=click.Path(resolve_path=True))
 @click.argument("workspace", type=click.Path())
-@click.option(
-    "-f",
-    "--fold-results",
-    type=click.Path(exists=True, resolve_path=True),
-    multiple=True,
-    help="fold output directories",
-)
-@click.option(
-    "-s",
-    "--single-results",
-    type=click.Path(exists=True, resolve_path=True),
-    multiple=True,
-    help="single result dirs",
-)
+@click.option("-f", "--fold-results", type=click.Path(exists=True, resolve_path=True), multiple=True, help="fold output directories")
+@click.option("-s", "--single-results", type=click.Path(exists=True, resolve_path=True), multiple=True, help="single result dirs")
 @click.option("--and-submit", is_flag=True, help="submit the condor jobs")
-def apply_all(datadir, arrname, outdir, workspace, fold_results=None, single_results=None, and_submit=False):
+def apply_all(
+    datadir,
+    arrname,
+    outdir,
+    workspace,
+    fold_results=None,
+    single_results=None,
+    and_submit=False,
+):
     """Generate BDT response arrays for all ROOT files in DATAIR"""
-    import glob, shutil
+    import glob
+    import shutil
     import pycondor
 
     if len(single_results) > 0 and len(fold_results) > 0:
@@ -548,9 +480,8 @@ def apply_all(datadir, arrname, outdir, workspace, fold_results=None, single_res
 
     ws = PosixPath(workspace).resolve()
 
-    outpath = PosixPath(outdir)
-    outpath_empty = len(os.listdir(outpath)) == 0
-    outpath.mkdir(exist_ok=outpath_empty)
+    outpath = PosixPath(outdir).resolve()
+    outpath.mkdir(exist_ok=True)
 
     datapath = PosixPath(datadir).resolve(strict=True)
     all_files = glob.glob(f"{datapath}/*.root")
@@ -600,14 +531,14 @@ def soverb(datadir, selections, use_tptrw):
     """
     from tdub.frames import raw_dataframe, apply_weight_tptrw, satisfying_selection
     from tdub.data import quick_files
-    from tdub.branches import minimal_branches
+    from tdub.branches import selection_branches
 
     with open(selections, "r") as f:
         selections = json.load(f)
 
     necessary_branches = set()
     for selection, query in selections.items():
-        necessary_branches |= minimal_branches(query)
+        necessary_branches |= selection_branches(query)
     necessary_branches = list(necessary_branches) + ["weight_tptrw_tool"]
 
     qf = quick_files(datadir)
@@ -623,6 +554,24 @@ def soverb(datadir, selections, use_tptrw):
         print(sel, s_df["weight_nominal"].sum() / b_df["weight_nominal"].sum())
 
 
+@cli.command("rex-plot", context_settings=dict(max_content_width=92))
+@click.argument("workspace", type=click.Path(exists=True))
+@click.option("-o", "outdir", type=str, default="auto")
+def rex_plot(workspace, outdir):
+    """Generate plots from TRExFitter WORKSPACE using METADATA."""
+    import tdub.rex
+    import tdub.config
+
+    if outdir == "auto":
+        outdir = PosixPath(workspace) / "matplotlib"
+    else:
+        outdir = PosixPath(outdir)
+    outdir.mkdir(exist_ok=True)
+    tdub.config.download_meta_table()
+    tdub.config.set_default_logy()
+    tdub.rex.plot_all_regions(workspace, outdir)
+
+
 def run_cli():
     import tdub.config
 
